package cn.ischoolcode.javabook.scene9;

/* 
 * 爱校码
 * http://www.ischoolcode.cn/
 *  
 * @author zcj
 */
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.ArrayList;
import java.io.*;

/**
 * 类MyServer是一个实现ActionListener接口的听用户操作的GUI类
 */

public class MyServer extends JFrame {
	private JTextArea messageArea = new JTextArea(); // 从服务器显示的信息
	private JScrollPane jsp = new JScrollPane(messageArea); // 滚动区域的信息区
	private ClientHandler ch;

	public MyServer(int port) {
		ch = new ClientHandler(this, port); // 创建一个ClientHandler对象
		messageArea = new JTextArea();
		messageArea.setEditable(false);
		jsp = new JScrollPane(messageArea);

		// 放置组件的容器
		Container con = getContentPane();
		con.setLayout(new BorderLayout());
		con.add(jsp, "Center");
		addWindowListener(closeWindow);
		setSize(600, 300);
		show();
	}

	WindowAdapter closeWindow = new WindowAdapter() {
		public void windowClosing(WindowEvent e) {

			System.exit(0);
		}
	};

	/* 用于设定一个新的标题 */
	public void newTitle(String host, int port, int clientCount) {

		setTitle("聊天服务器主机： " + host + " 侦听端口： " + port + " 客户数： " + clientCount);

	}

	/* 用于在消息区显示消息 */
	public void showMessage(String msg) {

		messageArea.append(msg + "\n");

	}

	/* Main方法 */
	public static void main(String[] args) {

		int port;
		try {
			port = Integer.parseInt(args[0]);
		} catch (Exception e) {
			port = 2000;
		}
		new MyServer(port);
	}
}
/**
* 类ClientHandler是一个线程类，在一个集合中放置连接客户端。 
* @author zcj
*/

class ClientHandler implements Runnable {

	public ServerSocket serverSocket;
	public int port; // 服务器监听端口
	public int clientCount; // 客户端数量
	public MyServer chatServer;
	private ArrayList clientArr; // 客户动态数组
	private Thread thread;

	/* 构造方法 */
	public ClientHandler(MyServer chatServer, int port) {
		this.chatServer = chatServer;
		this.port = port;
		try {
			serverSocket = new ServerSocket(port);
		} catch (Exception e) {
			System.err.println("不能创建套接字连接 " + e);
			System.exit(0);
		}
		clientCount = 0;
		try {
			chatServer.newTitle(serverSocket.getInetAddress().getLocalHost()
					.getHostName(), port, clientCount);
		} catch (Exception e) {
		}
		clientArr = new ArrayList();
		thread = new Thread(this);
		thread.start(); // 启动这个线程
	}

	/* 寻找新的客户，并将其添加到一个ArrayList */
	public void run() {
		while (true) {
			try {
				Socket socket = serverSocket.accept();
				chatServer.showMessage(socket.getInetAddress() + " 已连接");
				clientArr.add(new Client(socket, this));
			} catch (Exception e) {
			}
		}
	}

	/* 将消息发送给所有连接的客户端 */
	public synchronized void broadcast(String msg) {
		chatServer.showMessage(msg);
		for (int i = 0; i < clientArr.size(); i++) {
			Client client = (Client) clientArr.get(i);
			client.sendMessage(msg);
		}
	}

	// 从ArrayList列表中杀死并删除一个客户端
	public synchronized void killClient(Client c) {
		String client = c.socket.getInetAddress().toString();
		try {
			c.in.close();
			c.out.close();
			c.socket.close();
			for (int i = 0; i < clientArr.size(); i++) {
				if (((Client) clientArr.get(i)).equals(c)) {
					clientArr.remove(i);
				}
			}
			clientArr.trimToSize();
			clientCount--;
			chatServer.newTitle(serverSocket.getInetAddress().getLocalHost()
					.getHostName(), port, clientCount);
		} catch (Exception e) {
		}
		broadcast(client + " 已断开连接");
	}
} // 结束ClientHandler类

/**
 * 为每个连接的客户端创建一个客户端类的一个实例
 * 
 * @author zcj
 */

class Client extends Thread {

	public Socket socket;
	public ClientHandler ch;
	public BufferedReader in;
	public PrintWriter out;
	private String host;

	public Client(Socket socket, ClientHandler ch) {
		this.socket = socket;
		this.ch = ch;
		ch.clientCount++;
		try {
			ch.chatServer.newTitle(ch.serverSocket.getInetAddress()
					.getLocalHost().getHostName(), ch.port, ch.clientCount);
		} catch (Exception e) {
		}
		try {
			in = new BufferedReader(new InputStreamReader(
					socket.getInputStream()));
			host = socket.getInetAddress().getHostName();
			out = new PrintWriter(socket.getOutputStream(), true);
			out.println("欢迎来到 "
					+ ch.serverSocket.getInetAddress().getLocalHost()
							.getHostName());
		} catch (Exception e) {
		}
		this.start();
	}

	// 将消息发送到客户端对象
	public void sendMessage(String msg) {
		try {
			out.println(msg);
		} catch (Exception e) {
			ch.killClient(this);
		}
	}

	// 接收消息
	public void run() {
		String inStream = " ";
		try {
			while ((inStream = in.readLine()) != null) {
				ch.broadcast(socket.getInetAddress() + ": " + inStream);
				inStream = " ";
			}
			out.close();
			in.close();
			socket.close();
		} catch (Exception e) {
		}
		ch.killClient(this);
	}
} // 结束 Client类

